@@ -829,7 +829,8 @@ _wnck_deiconify (Window xwindow)
 
 void
 _wnck_close (Screen *screen,
-	     Window  xwindow)
+	     Window  xwindow,
+	     Time    timestamp)
 {
   XEvent xev;
   
@@ -840,7 +841,7 @@ _wnck_close (Screen *screen,
   xev.xclient.window = xwindow;
   xev.xclient.message_type = _wnck_atom_get ("_NET_CLOSE_WINDOW");
   xev.xclient.format = 32;
-  xev.xclient.data.l[0] = 0;
+  xev.xclient.data.l[0] = timestamp;
   xev.xclient.data.l[1] = 0;
   xev.xclient.data.l[2] = 0;
 
@@ -971,9 +972,14 @@ _wnck_change_workspace (Screen     *screen,
 
 void
 _wnck_activate (Screen *screen,
-		Window  xwindow)
+                Window  xwindow,
+                Time    timestamp)
 {
   XEvent xev;
+
+  if (timestamp == 0)
+    g_warning ("Received a timestamp of 0; window activation may not "
+               "function properly.\n");
   
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -982,9 +988,11 @@ _wnck_activate (Screen *screen,
   xev.xclient.window = xwindow;
   xev.xclient.message_type = _wnck_atom_get ("_NET_ACTIVE_WINDOW");
   xev.xclient.format = 32;
-  xev.xclient.data.l[0] = 0;
-  xev.xclient.data.l[1] = 0;
+  xev.xclient.data.l[0] = 2;
+  xev.xclient.data.l[1] = timestamp;
   xev.xclient.data.l[2] = 0;
+  xev.xclient.data.l[3] = 0;
+  xev.xclient.data.l[4] = 0;
 
   XSendEvent (gdk_display,
 	      RootWindowOfScreen (screen),
@@ -995,7 +1003,8 @@ _wnck_activate (Screen *screen,
 
 void
 _wnck_activate_workspace (Screen *screen,
-			  int     new_active_space)
+                          int     new_active_space,
+                          Time    timestamp)
 {
   XEvent xev;
   
@@ -1007,8 +1016,10 @@ _wnck_activate_workspace (Screen *screen,
   xev.xclient.message_type = _wnck_atom_get ("_NET_CURRENT_DESKTOP");
   xev.xclient.format = 32;
   xev.xclient.data.l[0] = new_active_space;
-  xev.xclient.data.l[1] = 0;
+  xev.xclient.data.l[1] = timestamp;
   xev.xclient.data.l[2] = 0;
+  xev.xclient.data.l[3] = 0;
+  xev.xclient.data.l[4] = 0;
 
   XSendEvent (gdk_display,
 	      RootWindowOfScreen (screen),
